/*  Copyright © 2015 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 10/12/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __math_lambert_h__
#define __math_lambert_h__ 1


/*   std includes   */

/*   project headers   */


namespace duct {
  
  
  /* Lambert's Function W_0(x)
   *
   * W_0(x) is the principal branch of the
   * implicit function defined by W e^W = x.
   *
   * -1/E < x < \infty
   */
  double lambert_W0(double);
  
  /* Lambert's Function W_{-1}(x)
   *
   * W_{-1}(x) is the principal branch of the
   * implicit function defined by W e^W = x.
   *
   * -1/E < x < \infty
   */
  double lambert_Wm1(double);

  
  /* Lambert's Function W_0(x)
   *
   * This function has to be called with log argument, thus
   * lambert_W0_lx(w) = W_0(exp(w)).
   *
   *  -\infty < w < \infty
   */
  double lambert_W0_lx(double);
}

#endif