/*  Copyright (C) 2012-2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 5/3/12.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef __proc_dyjets_h__
#define __proc_dyjets_h__ 1

//   std includes
#include <functional>

//   project headers
#include <bits/duct-model.h>
#include <bits/mc-shower-history.h>



namespace duct {
  
  
  
  class dyjets
  {
  public:
    /*    constructor     */
    dyjets(const model_ref&, double, double, double, double = 1.0, int = 2212, int = 2212,
           std::initializer_list<int> = {11, 13, 15},
           std::initializer_list<int> = {-5,-4,-3,-2,-1,1,2,3,4,5});
    
    
    /*  this operator generates a new hard event   */
    void generate(random_engine_type&, shower_history<hhc>&);
    
  private:
    /*   key of the incoming hadrons   */
    int _M_hadronA, _M_hadronB;
    
    /*   pointer to the pdfs   */
    const qcd::dglap *_M_pdf1, *_M_pdf2;
    
    /*   the model   */
    model_ref _M_mdl;
    
    /*    private members    */
    double _M_Ecm, _M_scalefac;
    std::vector<int> _M_lpts, _M_partons;
    
    /*    lambda expressions     */
    std::function<double(double)> _M_rhotau;
    std::function<double(double, double)> _M_generate_tau;
    
    double _M_select_flavors(shower_history<hhc>::iterator, double, double, double, double);
    double _M_process_qqll(double, double, int, int) const;
  };
  
}

#endif
